import { Dialog } from "@mui/material";
import { DialogTitle } from "@mui/material";
import { DialogContent } from "@mui/material";
import { DialogContentText } from "@mui/material";
import { DialogActions } from "@mui/material";
import { Button } from "@mui/material";
import { TextField } from "@mui/material";
import { useEffect, useState } from "react";
import { useContext } from "react";
import { LoginContext } from "./Context";
// import { TestContext } from "./Context";

interface RegisterData {
    username: string,
    password: string,
    captcha: string,
}

interface LoginData {
    username: string,
    password: string,
}

function sendRegister(data: RegisterData): void {
    console.log(JSON.stringify(data));
    fetch("/api/register", {
        method: "POST",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify(data)
    });
}

interface LoginResponse {
    username: string,
    token: string,
}

async function sendLogin(data: LoginData): Promise<LoginResponse> {
    // console.log(JSON.stringify(data));
    const response_promise = await fetch("/api/login", {
        method: "POST",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify(data)
    });

    const logindata = await response_promise.json();
    return (logindata as LoginResponse);
}

function LoginDialog(): JSX.Element {
    const [username, setUsername] = useState("");
    const [password, setPassword] = useState("");

    const loginCTX = useContext(LoginContext);

    const setLoggedInAs = (username: string, token: string): void => {
        loginCTX.setCurrentUser?.(username);
        loginCTX.setUserToken?.(token);
        handleClose();
    }

    // const setLogOut = (): void => {
    //     loginCTX.setCurrentUser?.(undefined);
    //     loginCTX.setUserToken?.(undefined);
    //     localStorage.removeItem("loginState");
    // }

    const handleClose = (): void => {
        loginCTX.setOpen?.(false);
        setUsername("");
        setPassword("");
    };

    // Check for localStorage login state
    useEffect((): void => {
        const loginState = JSON.parse(localStorage.getItem("loginState")||"{}");
        if (loginState.username && loginState.token) {
            setLoggedInAs(loginState.username, loginState.token);
        }
    }, []);

    const handleLogin = async (): Promise<void> => {
        const response = await sendLogin({ username: username, password: password });
        if (response && response.username && response.token) {
            setLoggedInAs(response.username, response.token);
            localStorage.setItem("loginState", JSON.stringify(response));
        }

    };

    const handleRegister = (): void => {
        sendRegister({ username: username, password: password, captcha: "test" });
    }

    return (
        <Dialog open={loginCTX.loginModalOpen} onClose={handleClose} sx={{ p: 2, top: "-40%" }}>
            <DialogTitle>Login</DialogTitle>
            <DialogContent>
                <DialogContentText>
                    Please enter your username and password.
                </DialogContentText>
                <TextField
                    autoFocus
                    margin="dense"
                    id="username"
                    label="Username"
                    type="text"
                    fullWidth
                    value={username}
                    onChange={(e): void => setUsername(e.target.value)}
                />
                <TextField
                    margin="dense"
                    id="password"
                    label="Password"
                    type="password"
                    fullWidth
                    value={password}
                    onChange={(e): void => setPassword(e.target.value)}
                />
            </DialogContent>
            <DialogActions>
                <Button onClick={handleLogin}>Login</Button>
            </DialogActions>
            <DialogActions>
                <Button onClick={handleRegister}>Register</Button>
            </DialogActions>
        </Dialog>
    );
}

export default LoginDialog;