import { Box, Container, Grid, Button, Paper, TextField, FormControl } from "@mui/material";
import { useEffect, useState } from "react";
import SendIcon from '@mui/icons-material/Send';
import CancelIcon from '@mui/icons-material/Cancel';
import { Post, PostCard } from "./Post";

function Primary(): JSX.Element {
    const [posts, setPosts] = useState<Post[]>([]);

    const refreshPosts = (): void => {
        fetch("/api/").then((response): void => {
            response.json().then((data): void => {
                setPosts(data);
            })
        })
    }

    useEffect((): void => {
        refreshPosts();
    }, [])

    return (
        <Container sx={{ p: 2, display: "flex", flexDirection: "column", alignItems: "center", backgroundColor: "background.default", minHeight: "60vh" }}>
            <PostInput newPostCallback={refreshPosts} />
            <PostContainer posts={posts} />
        </Container>
    )
}

function PostContainer({ posts }: { posts: Post[] }): JSX.Element {
    return (
        <Grid container justifyContent={"center"} spacing={2} pb={2}>
            {posts.map((p): JSX.Element => {
                return (
                    <Grid item xs={12} sm={12} md={6} lg={6} xl={6} key={p.uuid}>
                        <PostCard key={p.uuid} post={p} />
                    </Grid>
                )
            })}
        </Grid>
    )
}

interface NewPost {
    content: string,
    token: string,
}

function PostInput({ newPostCallback }: { newPostCallback: () => void }): JSX.Element {
    const [currentInput, setCurrentInput] = useState("");

    const handleSubmit = (): void => {
        if (currentInput) submitPostToServer();
    }

    const submitPostToServer = async (): Promise<void> => {
        const newPost: NewPost = { content: currentInput, token: "" }

        const response = await fetch("/api/", {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(newPost)
        })
        const data = await response.json();
        console.log(data);
        newPostCallback();
        setCurrentInput("");
    }

    return (
        <Box py={2} display={"flex"} flexDirection={"column"} width={1}>
            <Paper sx={{ p: 2, display: "flex", flexDirection: "column", alignContent: "center", alignItems: "center" }} elevation={4}>
                <FormControl fullWidth>
                    {/* <TextField sx={{pb:1}} size="small" id="outlined-basic" label="Title" variant="outlined" value={title} fullWidth onChange={(a): void => setTitle(a.target.value)} /> */}
                    <TextField multiline id="outlined-basic" label="Share your thoughts" variant="outlined" minRows={4} value={currentInput} fullWidth onChange={(a): void => setCurrentInput(a.target.value)} />
                    <Container disableGutters sx={{ mt:1, display: "flex", flexDirection: "row", justifyContent: "flex-end", alignItems: "center", width: "100%" }}>
                    <Button sx={{mr:1}} type="reset" startIcon={<CancelIcon/>} variant="outlined" onClick={handleSubmit}>Cancel</Button>
                    <Button type="submit" startIcon={<SendIcon />} variant="contained" onClick={handleSubmit}>Send</Button>
                    </Container>
                </FormControl>
            </Paper>
        </Box>
    )
}

export default Primary;